@@ -1,10 +1,9 @@
-local fs        = require 'bee.filesystem'
-local fsu       = require 'fs-utility'
 local json      = require 'json'
 local proto     = require 'proto'
 local lang      = require 'language'
 local util      = require 'utility'
 local workspace = require 'workspace'
+local scope     = require 'workspace.scope'
 
 local function errorMessage(msg)
     proto.notify('window/showMessage', {
@@ -14,40 +13,43 @@ local function errorMessage(msg)
     log.error(msg)
 end
 
+---@class config.loader
 local m = {}
 
-function m.loadRCConfig(filename)
-    local path = workspace.getAbsolutePath(filename)
+function m.loadRCConfig(uri, filename)
+    local scp  = scope.getScope(uri)
+    local path = workspace.getAbsolutePath(uri, filename)
     if not path then
-        m.lastRCConfig = nil
+        scp:set('lastRCConfig', nil)
         return nil
     end
     local buf = util.loadFile(path)
     if not buf then
-        m.lastRCConfig = nil
+        scp:set('lastRCConfig', nil)
         return nil
     end
     local suc, res = pcall(json.decode, buf)
     if not suc then
         errorMessage(lang.script('CONFIG_LOAD_ERROR', res))
-        return m.lastRCConfig
+        return scp:get('lastRCConfig')
     end
-    m.lastRCConfig = res
+    scp:set('lastRCConfig', res)
     return res
 end
 
-function m.loadLocalConfig(filename)
-    local path = workspace.getAbsolutePath(filename)
+function m.loadLocalConfig(uri, filename)
+    local scp  = scope.getScope(uri)
+    local path = workspace.getAbsolutePath(uri, filename)
     if not path then
-        m.lastLocalConfig = nil
-        m.lastLocalType = nil
+        scp:set('lastLocalConfig', nil)
+        scp:set('lastLocalType', nil)
         return nil
     end
-    local buf  = util.loadFile(path)
+    local buf = util.loadFile(path)
     if not buf then
         errorMessage(lang.script('CONFIG_LOAD_FAILED', path))
-        m.lastLocalConfig = nil
-        m.lastLocalType = nil
+        scp:set('lastLocalConfig', nil)
+        scp:set('lastLocalType', nil)
         return nil
     end
     local firstChar = buf:match '%S'
@@ -55,10 +57,10 @@ function m.loadLocalConfig(filename)
         local suc, res = pcall(json.decode, buf)
         if not suc then
             errorMessage(lang.script('CONFIG_LOAD_ERROR', res))
-            return m.lastLocalConfig
+            return scp:get('lastLocalConfig')
         end
-        m.lastLocalConfig = res
-        m.lastLocalType = 'json'
+        scp:set('lastLocalConfig', res)
+        scp:set('lastLocalType', 'json')
         return res
     else
         local suc, res = pcall(function ()
@@ -66,36 +68,38 @@ function m.loadLocalConfig(filename)
         end)
         if not suc then
             errorMessage(lang.script('CONFIG_LOAD_ERROR', res))
-            return m.lastLocalConfig
+            scp:set('lastLocalConfig', res)
         end
-        m.lastLocalConfig = res
-        m.lastLocalType = 'lua'
+        scp:set('lastLocalConfig', res)
+        scp:set('lastLocalType', 'lua')
         return res
     end
 end
 
 ---@async
-function m.loadClientConfig()
+---@param uri? uri
+---@return table
+function m.loadClientConfig(uri)
     local configs = proto.awaitRequest('workspace/configuration', {
         items = {
             {
-                scopeUri = workspace.uri,
+                scopeUri = uri,
                 section = 'Lua',
             },
             {
-                scopeUri = workspace.uri,
+                scopeUri = uri,
                 section = 'files.associations',
             },
             {
-                scopeUri = workspace.uri,
+                scopeUri = uri,
                 section = 'files.exclude',
             },
             {
-                scopeUri = workspace.uri,
+                scopeUri = uri,
                 section = 'editor.semanticHighlighting.enabled',
             },
             {
-                scopeUri = workspace.uri,
+                scopeUri = uri,
                 section = 'editor.acceptSuggestionOnEnter',
             },
         },
