@@ -23,10 +23,10 @@ return function (uri, callback)
             if not key then
                 return
             end
-            if config.get 'Lua.diagnostics.globals'[key] then
+            if config.get(uri, 'Lua.diagnostics.globals')[key] then
                 return
             end
-            if config.get 'Lua.runtime.special'[key] then
+            if config.get(uri, 'Lua.runtime.special')[key] then
                 return
             end
         end
@@ -83,7 +83,11 @@ return function (uri, callback)
             end
             table.sort(versions)
             if #versions > 0 then
-                message = ('%s(%s)'):format(message, lang.script('DIAG_DEFINED_VERSION', table.concat(versions, '/'), config.get 'Lua.runtime.version'))
+                message = ('%s(%s)'):format(message
+                    , lang.script('DIAG_DEFINED_VERSION'
+                    , table.concat(versions, '/')
+                    , config.get(uri, 'Lua.runtime.version'))
+                )
             end
         end
         cache[id] = {
